package cn.com.qmth.scancentral.service.impl;

import java.util.ArrayList;
import java.util.List;

import cn.com.qmth.scancentral.entity.SubjectEntity;
import cn.com.qmth.scancentral.enums.ImageCheckStatus;
import cn.com.qmth.scancentral.enums.UploadStatus;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.service.SubjectService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.scancentral.dao.StudentDao;
import cn.com.qmth.scancentral.service.ImageCheckService;
import cn.com.qmth.scancentral.service.ToolExportService;
import cn.com.qmth.scancentral.util.MathUtil;
import cn.com.qmth.scancentral.util.PageUtil;
import cn.com.qmth.scancentral.vo.imagecheck.ImageCheckDetailVo;
import cn.com.qmth.scancentral.vo.imagecheck.ImageCheckFailedQuery;
import cn.com.qmth.scancentral.vo.imagecheck.ImageCheckVo;

@Component
public class ImageCheckServiceImpl implements ImageCheckService {

    private static final Logger log = LoggerFactory.getLogger(ImageCheckServiceImpl.class);

    @Autowired
    private ToolExportService toolExportService;
    @Autowired
    private StudentDao studentDao;
    @Autowired
    private SubjectService subjectService;
    @Autowired
    private StudentService studentService;
    @Override
    public List<ImageCheckVo> list(Long examId) {
        if (examId == null) {
            throw new IllegalArgumentException("考试ID不能为空");
        }
        List<SubjectEntity> subjectList =subjectService.listByExamId(examId);
//        List<ImageCheckVo> result = studentDao.countImageCheck(examId);
        List<ImageCheckVo> result = new ArrayList<>();
        for (SubjectEntity subject: subjectList) {
            ImageCheckVo vo =new ImageCheckVo();
            vo.setSubjectCode(subject.getCode());
            vo.setSubjectName(subject.getName());
            vo.setTotalCount(studentService.countByExamIdAndSubjectCode(examId,subject.getCode()));
            List<ImageCheckStatus> imageCheckStatusList = new ArrayList<>();
            List<UploadStatus> fileUploadStatusList = new ArrayList<>();
            vo.setFailedCount(studentService.getImageCheckFailedCount(examId,subject.getCode()));
            vo.setDisposedCount(studentService.getImageCheckDisposedCount(examId,subject.getCode()));
            if (vo.getTotalCount() == null || vo.getTotalCount() == 0) {
                vo.setImageCheckProgress(null);
            } else {
                double progress = MathUtil.percentage(vo.getDisposedCount(), vo.getTotalCount());
                vo.setImageCheckProgress(progress);
            }
            result.add(vo);
        }

        return result;
    }

    @Override
    public PageResult<ImageCheckDetailVo> failedPage(ImageCheckFailedQuery query) {
        if (query.getExamId() == null) {
            throw new IllegalArgumentException("考试ID不能为空");
        }
        if (StringUtils.isBlank(query.getSubjectCode())) {
            throw new IllegalArgumentException("科目代码不能为空");
        }
        IPage<ImageCheckDetailVo> page = studentDao.failedPage(new Page<>(query.getPageNumber(), query.getPageSize()),
                query);

        for (ImageCheckDetailVo vo : page.getRecords()) {
            vo.setFailed(vo.getFailedCount() > 0);
        }

        return PageUtil.of(page);
    }

    /**
     * 图片检查
     */
    @Override
    public void check(Long examId) {
        log.warn("图片检查开始... examId:{}", examId);
        long startTime = System.currentTimeMillis();

        int batchSize = 1000;
        int checkedSize = 0;
        Long startId = 0L;
        while (true) {
            List<Long> studentIds = studentDao.queryImageCheckWaitingList(examId, startId, batchSize);
            if (CollectionUtils.isEmpty(studentIds)) {
                break;
            }

            for (Long studentId : studentIds) {
                toolExportService.studentCheck(studentId);
            }

            checkedSize += studentIds.size();
            log.info("图片检查中... examId:{} checkedSize:{} startId:{}", examId, checkedSize, startId);

            startId = studentIds.get(studentIds.size() - 1);
        }

        long cost = (System.currentTimeMillis() - startTime) / 1000;
        log.warn("图片检查结束... examId:{} checkedSize:{} cost:{}s", examId, checkedSize, cost);
    }

}
